package segmap

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Segmap

This library offers an implementation of segment maps. A segment map represents a thinning, an increasing function of the natural numbers to the natural numbers. A thinning can serve as a renaming in a system where variables are represented as de Bruijn indices.

Documentation

The library offers just one module, Segmap.

Installation and Usage

Type opam install segmap.

In your dune file, add (libraries segmap) to the description of your library or executable.